#version 430 core
//
//	Copyright (c) 2015, Kin-Ming Wong and Tien-Tsin Wong
//  All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//	Redistributions of source code must retain the above copyright notice,
//	this list of conditions and the following disclaimer.
//
//	Redistributions in binary form must reproduce the above copyright notice,
//	this list of conditions and the following disclaimer in the documentation
//	and/or other materials provided with the distribution.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
//	IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
//	INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
//	BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
//	OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
//	OF THE POSSIBILITY OF SUCH DAMAGE.
//
//	Please cite our original article
//	'Edge-preserving smoothing filter for particle-based rendering'
//	in Game Engine Gems 3 if you use any part of the following code.
//
layout(binding = 0) uniform sampler2D sat_I2;
layout(binding = 1) uniform sampler2D sat_I;

layout(location = 0) out vec4 Ak;
layout(location = 1) out vec4 Bk;

uniform int r;
uniform float epsilon;

void main(void)
{

	vec2	s = 1.0 / textureSize(sat_I2, 0);
	vec2	coord = gl_FragCoord.xy;

	int		r1 = -r - 1;
	vec2 P0 = s * (coord + vec2(r1, r1));
	vec2 P1 = s * (coord + vec2(r1,  r));
	vec2 P2 = s * (coord + vec2( r, r1));
	vec2 P3 = s * (coord + vec2( r,  r));

	float	patchWidth = r + r + 1;
	float	omega = 1.0 / (patchWidth * patchWidth);

	vec3	a, b, c, d;

	a = textureLod(sat_I2, P0, 0).rgb;
	b = textureLod(sat_I2, P1, 0).rgb;
	c = textureLod(sat_I2, P2, 0).rgb;
	d = textureLod(sat_I2, P3, 0).rgb;

	//	A = mean of I^2
	vec3	ui2 = vec3(omega) * (a - b - c + d);

	a = textureLod(sat_I, P0, 0).rgb;
	b = textureLod(sat_I, P1, 0).rgb;
	c = textureLod(sat_I, P2, 0).rgb;
	d = textureLod(sat_I, P3, 0).rgb;

	//	u = mean of I
	vec3	ui = vec3(omega) * (a - b - c + d);

	vec3	A	= ui2 - (ui * ui);
	vec3	ak	= A / (vec3(epsilon) + A);
	vec3	bk	= ui * (vec3(1.0) - ak);

	Ak = vec4(ak.rgb, 1.0);
	Bk = vec4(bk.rgb, 1.0);

}
